#include <rFlex.h>
#include <rFlexP.h>

/* ============================================================ */
/* ============================================================ */

static int
irRpt(struct reflex *reflex, unsigned char id,
	 int len, unsigned char *buffer)
{
  int ii,jj;
  int retval;
  int count;
  int timeStamp[2];
  int numBoards;
  int index = 0;
  unsigned char    flag[256];
  unsigned short irValue[256];

  /* should check on len and opcode */

  ii = 0;
  count = 0;


  numBoards = pktGetChar (buffer, &index);
  for (ii= 0; ii < numBoards; ii++)
    {
      timeStamp[ii] = pktGetLong (buffer, &index);
      for (jj= 0; jj<8; jj++)
	irValue[ii*8+jj] = pktGetChar (buffer, &index);
      flag[ii] = pktGetChar (buffer, &index);
    }
  
  if (reflex->ir->rpt) {
    retval = reflex->ir->rpt(reflex, id, timeStamp, flag, irValue);
  }

  return(retval);
}
  
/*------------------------------------------------------------*/

static int
rptReq(struct reflex *reflex, unsigned char id,
       int triggerDelay, int fireDelay, int retryDelay, int setDelay, 
       int goneTimeout, unsigned char runMode)
{
  int len;
  unsigned char buffer[256];
  unsigned char opcode;
  unsigned char port;
  int retval;

  len = 0;

  opcode = IR_RUN;
  port   = reflex->ir->port;

  pktAddLong (buffer, &len, triggerDelay);
  pktAddLong (buffer, &len, fireDelay);
  pktAddLong (buffer, &len, retryDelay);
  pktAddLong (buffer, &len, setDelay);
  pktAddLong (buffer, &len, goneTimeout);
  pktAddChar (buffer, &len, runMode);

  retval = pktWrite(reflex->pktPort, port, id, opcode, buffer, len);
  return (retval);
}
  
/*------------------------------------------------------------*/

static int
rptRpl (struct reflex* reflex, unsigned char id,
	int len, unsigned char *buffer)
{
  int ii;
  int retval;
  int timeStamp;

  /* should check on len and opcode */

  ii = 0;

  retval    = pktGetLong(buffer, &ii);
  timeStamp = pktGetLong(buffer, &ii);

  if (reflex->ir->rptRpl) {
    retval = reflex->ir->rptRpl(reflex, retval, id, timeStamp);
  }
  return(retval);
}

/* ============================================================ */
/* ============================================================ */
/* ============================================================ */

static int
handlePort(struct reflex* reflex,
	   int size, unsigned char *buffer,
	   unsigned char id, unsigned char opcode)
{
  int retval;

  switch (opcode) {
    
  case IR_RUN:
    retval = rptRpl       (reflex, id, size, buffer);
    break;
    
  case IR_REPORT:
    retval = irRpt       (reflex, id, size, buffer);
    break;

  default:
    retval = 0;
    if (reflex->ir->unknownRpl) {
      retval = reflex->ir->unknownRpl(reflex, id, size, buffer, opcode);
    }
  }

  return(retval);
}

/* ============================================================ */
/* ============================================================ */

static int
destroy (struct reflexIR *ir) {

  if (ir) free(ir);
  return(0);
}

/* ------------------------------------------------------------ */

struct reflexIR *
reflexIRNew (int port) {
  struct reflexIR *ir;

  ir = malloc(sizeof(*ir));
  if (!ir) return(NULL);
  memset(ir, 0, sizeof(*ir));

  ir->port       = port;

  ir->rptReq     = rptReq;

  ir->handlePort = handlePort;
  ir->destroy    = destroy;

  return(ir);
}
